﻿using System;

namespace CySoft.Collections.Geometry
{
	/// <summary>
	/// Class for tree with Weighted ManhattanKDTree distancing.
	/// </summary>
	/// <typeparam name="T">Value type</typeparam>
	public class WeightedManhattanKDTree<T> : WeightedKDTreeBase<T>
	{
		public WeightedManhattanKDTree(int dimensions, double[] weights)
			: base(dimensions, weights, null)
		{
		}

		public WeightedManhattanKDTree(int dimensions, double[] weights, int sizeLimit)
			: base(dimensions, weights, sizeLimit)
		{
		}

		protected override double PointDist(double[] p1, double[] p2)
		{
			double d = 0;

			for (int i = 0; i < p1.Length; i++) {
				double diff = (p1[i] - p2[i]);
				if (!Double.IsNaN(diff)) {
					d += ((diff < 0) ? -diff : diff) * _weights[i];
				}
			}

			return d;
		}

		protected override double PointRegionDist(double[] point, double[] min, double[] max)
		{
			double d = 0;

			for (int i = 0; i < point.Length; i++) {
				double diff = 0;
				if (point[i] > max[i]) {
					diff = (point[i] - max[i]);
				} else if (point[i] < min[i]) {
					diff = (min[i] - point[i]);
				}

				if (!Double.IsNaN(diff)) {
					d += diff * _weights[i];
				}
			}

			return d;
		}
	}

}
