package net.robowiki.knn.implementations;
import net.robowiki.knn.util.KNNPoint;
import java.util.*;
// Namespace I use for my kd-tree
import dn.j1.algorithm.nearestneighbours.*;

public class DuynTreeImpl extends KNNImplementation {
  final BucketKdTree<KNNPointAdapter> tree;
  public DuynTreeImpl(final int dimension) {
    super(dimension);
    tree = new BucketKdTree<KNNPointAdapter>(10);
  }

  @Override public void
  addPoint(final double[] location, final String value) {
    tree.add(new KNNPointAdapter(location, value));
  }

  @Override public String
  getName() {
    return "duyn's Bucket kd-tree";
  }

  @Override public KNNPoint[]
  getNearestNeighbors(final double[] location, final int size) {
    final SortedMap<Double, List<KNNPointAdapter>> results = tree.search(location, size);
    final List<KNNPoint> justPoints = new LinkedList<KNNPoint>();
    for(final Map.Entry<Double, List<KNNPointAdapter>> entry : results.entrySet()) {
      final double distance = entry.getKey();
      for(final KNNPointAdapter pt : entry.getValue()) {
        justPoints.add(new KNNPoint(pt.value, distance));
      }
    }
    final KNNPoint[] retVal = new KNNPoint[justPoints.size()];
    return justPoints.toArray(retVal);
  }

  class KNNPointAdapter extends Exemplar {
    public final String value;
    public KNNPointAdapter(final double[] coords, final String value) {
      super(coords);
      this.value = value;
    }
  }
}
