package net.robowiki.knn.implementations;

import java.util.List;

import net.robowiki.knn.util.KNNPoint;
import ags.utils.dataStructures.trees.firstGenKD.*;

/**
 * @author Nat Pavasant
 */
public class Rednaxela1stGenTreeKNNSearch extends KNNImplementation {
	private KDTree<Point> tree;

	public Rednaxela1stGenTreeKNNSearch(int dimension) {
		super(dimension);
		tree = new KDTree<Point>(dimension);
	}

    @Override
	public void addPoint(double[] location, String value) {
		tree.add(new Point(value, location));
	}

	@Override
	public KNNPoint[] getNearestNeighbors(double[] location, int size) {
		List<Point> lists = tree.NearestNeighbors(new HyperPoint(location),
				size);
		KNNPoint[] data = new KNNPoint[size];
		for (int i = 0; i < size; i++) {
			data[i] = new KNNPoint(lists.get(i).value, getDistance(location, lists.get(i).location));
		}
		return data;
	}

	private static class Point implements KDEntry {
		private static final long serialVersionUID = 1L;

		public final String value;
		public final double[] location;
		public final HyperPoint point;

		public Point(String value, double[] location) {
			super();
			this.value = value;
			this.location = location;
			point = new HyperPoint(location);
		}

		public HyperPoint getPosition() {
			return point;
		}

	}

	@Override
	public String getName() {
		return "Rednaxela's Old Bucket kd-tree";
	}
}
